
% assess.pl
:- dynamic data1/1, context/3. 
% Pre is e.g. foil-rules-exp1-cn-, int is 1

ont1(Arg1):- superclass(Arg1,_).
ont2(Arg2):- 
	relation_under_test(Reln),
        arg2(Reln,Arg2).
word(W):-  posSent(_,_,W).
pos(P):-   posSent(_,P,_).
sent(S):-  posSent(S,_,_).

p_fact([Sent,[Reln,Arg1]]):-	
	relation_under_test(Reln),
	relation_under_test_arity(unary),
        sent(Sent),
	ont1(Arg1),
	Q =.. [Reln,Sent,Arg1],
	call(Q).

p_fact([Sent,[Reln,Arg1,Arg2]]):-	
	relation_under_test(Reln),
	relation_under_test_arity(binary),
        sent(Sent),
	ont1(Arg1),
	arg2(Reln,Arg2),  % may leave Arg2 unbound -> problem with negation in rule preconds
	Q =.. [Reln,Sent,Arg1,Arg2],
	call(Q).

p_fact([Sent,[Reln,Arg1,Arg2]]):-	
	relation_under_test(Reln),
	relation_under_test_arity(binary),
        sent(Sent),
	ont1(Arg1),
        word(Arg2),
	arg2(Reln,Arg2),
	Q =.. [Reln,Sent,Arg1,Arg2],
	call(Q).

/*
% precision - not using ontology constraints
p_fact([Sent,[Reln,Arg1,Arg2]]):-	
	relation_under_test(Reln),
	relation_under_test_arity(binary),
	Q =.. [Reln,Sent,Arg1,Arg2],
	call(Q),
        ground(Arg1),
        ground(Arg2).
*/

score_all(P):-
    ilp_system(foil),
    go(testing,POSList,_),!,
    score_all(POSList,P,1,[],[],[],[]),!.

score_all(P):-
    ilp_system(lime),
    go(testing,POSList,_),!,
    mod_words_lime(POSList,POSList1),
    score_all(POSList1,P,1,[],[],[],[]),!.

score_all(P):-
    ilp_system(golem),
    go(testing,POSList,_),!,
    mod_words_golem(POSList,POSList1),
    score_all(POSList1,P,1,[],[],[],[]),!.

score_all(P):- nl,nl,write('**** scoring failed ****  '),write(P),nl,nl,!.

score_all(_,Pre,11,RSS,RTT,PSS,PTT):-!,
    assertz(data1([Pre,RSS,RTT,PSS,PTT])),
    nl,write('data(\''),write(Pre),write('\','),
    nl,write('\%  recall:'),
    nl,write(RSS),write(','),
    nl,write(RTT),write(','),
    nl,write('\%  precision:'),
    nl,write(PSS),write(','),
    nl,write(PTT),write(').'),!.

score_all(POSList,Pre,INT,RSS,RTT,PSS,PTT):- 
        clean_and_consult(Pre,INT),!,
        score(POSList,INT,[PP,PS,PT],[RP,RS,RT]),
	nl,nl,
	nl,write('-----------------------------------'),
	nl,write(' Scores       (%)'),
	nl,write('  Recall:     '),write(RP),
        write(' ('),write(RS),write('/'),write(RT),write(')'),
	nl,write('  Precision:  '),write(PP),
        write(' ('),write(PS),write('/'),write(PT),write(')'),
	nl,write('-----------------------------------'),
	INT2 is INT + 1,!,
	score_all(POSList,Pre,INT2,[RS|RSS],[RT|RTT],[PS|PSS],[PT|PTT]).


clean_and_consult(Pre,INT):-
    relation_under_test(RELN),
    relation_under_test_arity(AC),
    convert_arity(AC,ACInt),
    Arity is ACInt + 1, % n+1 place reln
    abolish(RELN,Arity),
    concat(Pre,INT,FFILE),
    concat(FFILE,'.pl',PF),
    clean_c(FFILE,PF),
    consult(PF),!.

convert_arity(unary,1).
convert_arity(binary,2).
convert_arity(ternary,3).

% score(Precision,Recall).
/*
score(P,R):-
	go(testing,POSList0,_),!,
        make_uniq(POSList0,POSList),
	retractall(posSent(_,_,_)),
	assert_all_possent(POSList),!,
	extract_target(testing,TGT),!,
	precision(POSList,TGT,P),
	recall(TGT,R).
*/
score(POSList00,INT,P,R):-
        testing_POS(INT,POSList00,POSList0),
        make_uniq(POSList0,POSList),
        nl,nl,write('Test Set: '),write(INT),
	retractall(posSent(_,_,_)),
	retractall(context(_,_,_)),
	assert_all_possent(POSList),!,
        get_context(POSList,CX,_),
        assert_all_possent(CX),!,
	extract_target(testing,TGT0),!,
        testing_TGT(INT,TGT0,TGT),
	precision(POSList,TGT,P),
	recall(TGT,R).

score:- [foil_rules],
        score([PP,PS,PT],[RP,RS,RT]),
	nl,nl,
	nl,write('-----------------------------------'),
	nl,write(' Scores       (%)'),
	nl,write('  Recall:     '),write(RP),
        write(' ('),write(RS),write('/'),write(RT),write(')'),
	nl,write('  Precision:  '),write(PP),
        write(' ('),write(PS),write('/'),write(PT),write(')'),
	nl,write('-----------------------------------').


score(INT):- [foil_rules],
        score(INT,[PP,PS,PT],[RP,RS,RT]),
	nl,nl,
	nl,write('-----------------------------------'),
	nl,write(' Scores       (%)'),
	nl,write('  Recall:     '),write(RP),
        write(' ('),write(RS),write('/'),write(RT),write(')'),
	nl,write('  Precision:  '),write(PP),
        write(' ('),write(PS),write('/'),write(PT),write(')'),
	nl,write('-----------------------------------').


recall(TGT,[P,S,Total]):-
	recall_score(TGT,S,Total),
	p_cent(S,Total,P).

recall_score([],0,0):-!.
recall_score([[_,[Reln|_]]|L],M,X):-
	not(relation_under_test(Reln)),
	recall_score(L,M,X),!.

recall_score([[Sent,[Reln,Arg1]]|L],N,Y):-
	relation_under_test(Reln),
	Q =.. [Reln,Sent,Arg1],
	call(Q),!,
	recall_score(L,M,X),!,
	N is M + 1,
	Y is X + 1.
	
recall_score([[Sent,[Reln,Arg1,Arg2]]|L],N,Y):-
	relation_under_test(Reln),
	Q =.. [Reln,Sent,Arg1,Arg2],
	call(Q),!,
	recall_score(L,M,X),!,
	N is M + 1,
	Y is X + 1.
	
recall_score([[_,[Reln|_]]|L],N,Y):-
	relation_under_test(Reln),
	recall_score(L,N,X),!,
	Y is X + 1.
	
assert_all_possent([]):-!.
assert_all_possent([A|I]):-
	assertz(A),!,
	assert_all_possent(I).

% provable+correct/provable

precision(_,TGT,[P,S,Total]):-
        findall(I,p_fact(I),IL),
	make_uniq(IL,ILL),
	provable_correct(TGT,ILL,PC),
	length(PC,S),
	length(ILL,Total),
	p_cent(S,Total,P).

p_cent(0,0,0):-!.
p_cent(S,Total,P):-
	P is (S * 100) / Total.

provable_correct(_,[],[]):-!.
provable_correct(TGT,[I|J],[I|R]):-
	member(I,TGT),!,
	provable_correct(TGT,J,R).
provable_correct(TGT,[_|J],R):-
	provable_correct(TGT,J,R).

get_sent([],I,I):-!.
get_sent([posSent(S,_,_)|I],X,R):-
	member(S,X),!,
	get_sent(I,X,R),!.

get_sent([posSent(S,_,_)|I],X,R):-
	get_sent(I,[S|X],R).

count_tgt(_,_,11,[]):-!.
count_tgt(testing,R,X,[Y|Res]):-
	extract_target(testing,TGT0),!,
        testing_TGT(X,TGT0,TGT),
	count_tgts_(R,TGT,Y),!,
	XX is X + 1,
	count_tgt(testing,R,XX,Res).

count_tgt(training,R,X,[Y|Res]):-
	extract_target(training,TGT0),!,
        training_TGT(X,TGT0,TGT),
	count_tgts_(R,TGT,Y),!,
	XX is X + 1,
	count_tgt(training,R,XX,Res).



count_tgts_(_,[],0):-!.
count_tgts_(R,[[_,[R|_]]|L],Z):-
	count_tgts_(R,L,Y),
	Z is Y + 1.
count_tgts_(R,[_|L],Z):-
	count_tgts_(R,L,Z).
    
    

% make the same changes to words from test texts that
% were made in training, except don't change concepts to soConcept

mod_words_lime([],[]):-!.
mod_words_lime([posSent(S,P,A)|I],[posSent(S,P,A)|R]):-
    number(A),!,
    mod_words_lime(I,R),!.

mod_words_lime([posSent(S,P,A)|I],[posSent(S,P,B)|R]):-
    name(A,AN),
    delete(AN,45,ANN),    % remove any -
    delete(ANN,95,ANNN),  % remove any _
    lime_rename(ANNN,B),
    mod_words_lime(I,R),!.


lime_rename([X|A],B):-
	is_number(X),
	not_all_numerical(A),
	name(B0,[X|A]),
        concat(no,B0,B).
lime_rename(A,B):-
    name(B,A).

mod_words_golem([],[]):-!.
mod_words_golem([posSent(S,P,A)|I],[posSent(S,P,B)|R]):-
    name(A,AN),
    delete(AN,45,ANN),    % remove any -
    delete(ANN,95,ANNN),  % remove any _
    golem_rename(ANNN,B),
    mod_words_golem(I,R),!.

golem_rename([X|A],B):-
	is_number(X),
	not_all_numerical(A),
	hyphenate_word([X|A],AA),
	name(B,AA).
golem_rename(A,B):-
    name(B,A).
/*
assert_types:-
    findall(I,posSent(_,I,_),IL),
    findall(J,posSent(_,J,_),JL),
    make_uniq(IL,ILL),
    make_uniq(JL,JLL),
    assert_word(ILL),
    assert_pos(JLL).

assert_word([]):-!.
assert_word([I|L]):-
       assertz(word(I)),
       assert_word(L).

assert_pos([]):-!.
assert_pos([I|L]):-
       assertz(pos(I)),
       assert_pos(L).
*/
