

% clean_code.pl




clean_c(F,OPF):-
        ilp_system(foil),
	readin_xml(F,T),
	flatten(T,U),
	relation_under_test(R),
	concat(R,':',RR),
	scan_to(RR,U,VV),
	scan_to1(R,VV,VVV),
	clip_from('Time',VVV,V),
	swp_tkns(V,W),
	tell(OPF),
	wr_code(W),
	told.

clean_c(F,OPF):- 
        ilp_system(lime),
	readin_xml(F,T),
	flatten(T,U),
	relation_under_test(R),
	scan_to1(R,U,VV),
	clip_from('Seconds',VV,V),
	swp_tkns(V,W),
        gen_new_assns(W,X),
        append(W,X,WW),
	tell(OPF),
	wr_code(WW),
	told.

clean_c(F,OPF):- 
        ilp_system(golem),
	readin_xml(F,T),
	flatten(T,V),
	swp_tkns(V,W),
	tell(OPF),
	wr_code(W),
	told.

clean_c(_,OPF):-
	tell(OPF),
	wr_code,
	told.

load_all(I):-
    rep_clean_c(I,1).

rep_clean_c(_,11):-!.
rep_clean_c(F,INT):-
	concat(F,INT,FFile),
	concat(FFile,'.pl',PF),
	clean_c(FFile,PF),!,
	INT2 is INT + 1,
	consult(PF),
	rep_clean_c(F,INT2).

wr_code:-
    relation_under_test(R),
    relation_under_test_arity(binary),
    nl,write(R),write('(aa_,aa_,aa_).').

wr_code:-
    relation_under_test(R),
    relation_under_test_arity(unary),
    nl,write(R),write('(aa_,aa_).').

wr_code([]):-!.
wr_code(['.'|B]):-
	write('.'),nl,
	wr_code(B).
wr_code([A|B]):-
	write(A),
	wr_code(B).

swp_tkns([],[]):-!.

swp_tkns([A,'<','>',B|L],[not,'(',A,'=',B,')'|M]):-
    swp_tkns(L,M).

swp_tkns([':-'|L],[':-'|M]):-
	swp_tkns(L,M).
swp_tkns(['\%'|L],['\'\%\''|M]):-
	swp_tkns(L,M).

swp_tkns([A|L],[B|M]):-
	name(A,[X|AN]),
	is_uppercase(X),
        not_all_numerical(AN),
	length(AN,ANL),
	ANL > 1,	% A > 2
	concat('\'',A,AA),
	concat(AA,'\'',B),!,
	swp_tkns(L,M).

swp_tkns([A|L],[B|M]):-
	name(A,[115,111,X|AN]), % soConcept
	is_uppercase(X),
        name(B0,[X|AN]),
	concat('\'',B0,AA),
	concat(AA,'\'',B),!,
	swp_tkns(L,M).

swp_tkns([A|L],M):-
	name(A,AN),
        append(AH,[58,45|AT],AN), % :-
        name(B,AH),
        name(C,AT),
	swp_tkns([B,':-',C|L],M).

swp_tkns([A|L],[B|M]):-
	name(A,AN),
	member(45,AN),
	concat('\'',A,AA),
	concat(AA,'\'',B),!,
	swp_tkns(L,M).

swp_tkns([A|L],[B|M]):-
	name(A,AN),
	member(39,AN),  % '
        esc_quote([],AN,B),
	swp_tkns(L,M).

	
swp_tkns([A|L],[A|M]):-
	swp_tkns(L,M).

esc_quote(A,[39|B],C):-!,
    reverse(A,AA),
    name(AN,AA),
    name(BN,B),
    concat('\'',AN,ANN),
    concat(ANN,'\\\'',ANNN),
    concat(ANNN,BN,BNN),
    concat(BNN,'\'',C).
    
esc_quote(A,[X|B],C):-
    esc_quote([X|A],B,C).
	      

scan_to(R,[R|V],V):-!.
scan_to(R,[_|U],V):-
	scan_to(R,U,V).

scan_to1(R,[R|V],[R|V]):-!.
scan_to1(R,[_|U],V):-
	scan_to1(R,U,V).

clip_from(R,[R|_],[]):-!.
clip_from(R,[A|U],[A|V]):-
	clip_from(R,U,V).


gen_new_assns(L,New):-
    find_preds(L,[ont1,ont2,word,sent,pos,posSent,context,atmosphericConcentration,'atmosphericConcentration_Qual', changeInAtmosphericConcentration],M),
    construct_asns(M,New),!.

find_preds([],_,[]):-!.
find_preds([P,'('|L],PL,[P|R]):-
    not(member(P,PL)),
    find_preds(L,PL,R).

find_preds([_|L],PL,R):-
    find_preds(L,PL,R).

construct_asns([],[]):-!.
construct_asns([A|L],['   ',A,'(',B,')','.'|R]):-
    un_name(A,B),
    construct_asns(L,R).

% for '2.2' and no2 change to 2.2 and 2; for no2prime - no change
% as actual nos in posSent will not have the no prefix

un_name(A,B):-
    name(A,[39|L]), % '
    append(BB,[39],L),
    name(B,BB),
    number(B).

un_name(A,B):-
    name(A,[110, 111|L]), % no
    name(B,L),
    number(B).

un_name(A,A).
