
% generateILP.pl


:- dynamic count/1.

% test with: test.
% generate all 10 foil files

gen(I):- 
    go(training,E,WordFr),!,  
    gen(I,E,WordFr).

/*
gen(11):-!.
gen(I):- nl,write('Data Set '),write(I),writeILP(I),!,J is I + 1, gen(J).

*/

gen(11,_,_):-!.
gen(I,E,WordFr):- nl,write('Data Set '),write(I),
    writeILP(I,E,WordFr),!,J is I + 1, gen(J,E,WordFr).



writeILP:-  
	go(training,E),!,   
	write_to_ILP(E,0).
	
writeILP(I):-  
	go(training,E,WordFr),!,  
        training_POS(I,E,F),!,
	write_to_ILP(F,I,WordFr),!.
	
writeILP(I,E,WordFr):-  
        training_POS(I,E,F),!,
	write_to_ILP(F,I,WordFr),!.
	
%% the main write predicate  write_to_ILP/1
%% requires setup to have been executed

write_to_ILP(POSList000,INT,WordFr):-
	relation_under_test(RELN), 
	get_context(POSList000,CONTEXT0,_),
	make_uniq(POSList000,POSList00),
	min_word_frequency(Min),
        extract_target(training,TGT0),
        training_TGT(INT,TGT0,TGT1),

	modify_on_frequency(POSList00,Min,WordFr,POSList0),

	learning_input(Policy),
	modify_posList(Policy,POSList0,TGT1,POSList1),

	modify_sent_no(POSList1,POSList),
	modify_sent_no(TGT1,TGT),
        
	check_context(CONTEXT0,POSList,CONTEXT1),
	make_uniq(CONTEXT1,CONTEXT),
        concat('ilp-file',INT,FOILFILE),

	get_negated_tgts(TGT,POSList,_,NegWords),
	getTypes(sent,POSList,[],Stype0),
	make_uniq(Stype0,Stype),	

	getTypes(pos,POSList,[],POStype0),
	make_uniq(POStype0,POStype),	

        get_ontol_terms_arg2(TGT,ARG2Terms),
	arg2_list(RELN,ARG2Terms,ARG2Terms0),

	get_ontol_terms(TGT,OTerms),
        get_txt_terms(OTerms,TXTTerms),
	getTypes(word,POSList,TXTTerms,Wtype0),
	append(Wtype0,ARG2Terms0,Wtype1),
	append(Wtype1,[],Wtype2),
	make_uniq(Wtype2,Wtype),

	get_ontology(POSList,TGT,SUPER),

        ontol_terms_list(OTL),

	append(ARG2Terms0,NegWords,ARG2Terms1),
	make_uniq(ARG2Terms1,ARG2Terms2),

        write_ILP(FOILFILE,Stype,POStype,Wtype,OTL,ARG2Terms2,TGT,SUPER,OTerms,
		CONTEXT,POSList),
	   nl,
	   count_tgt_reln(RELN,TGT1,TGTLen),
	   write('No. Target Relations:          '),write(TGTLen),
	   nl,length(POSList00,POSList00Len),
	   write('No. Relations from Text        '),nl,
	   write(' Initial:                      '),write(POSList00Len),
           nl,length(POSList0,POSList0Len),
	   write(' After Frequency Modification: '),write(POSList0Len),
           nl,length(POSList1,POSList1Len),
	   write(' After Sentence Selection:     '),write(POSList1Len),
           nl,length(Stype,StypeLen),
	   write('No. Sentences:                 '),write(StypeLen),
           nl,length(POStype,POStypeLen),
	   write('Size of POS Set:               '),write(POStypeLen),
           nl,length(Wtype,WtypeLen),
	   write('Size of Word Set:              '),write(WtypeLen),
	   report_txt_mapping.


get_ontology(_,TGT,SUPER):-
	add_ontology(true),!,
	relation_under_test_arg1(ARG1),
        relation_under_test_concept(CON),
        superclass(CON,SCON),
        occurring_subclass(ARG1,TGT,SUPER3),
	make_uniq([superclass(CON,SCON)|SUPER3],SUPER).

get_ontology(_,_,[]):-!.

get_superclass_terms([],[CON]):-relation_under_test_concept(CON),!.
get_superclass_terms([superclass(I,J)|K],[I,J|L]):-
	get_superclass_terms(K,L).



report_txt_mapping:-
	use_txt_mapping(true),	
        nl,write('Using ontology to text mapping').
report_txt_mapping.


modify_on_frequency(POSList,Min,WordFList,POSList1):-

%	find_word_frequency([],POSList,WordFList),

        length(POSList,NoWords),
        calc_min(Min,NoWords,MinNo),
	delete_words(MinNo,WordFList,POSList,POSList1).
delete_words(1,_,R,S):-!,  % as delete_all changes actual numbers to cdPOS
	delete_all(R,'___',S),!. % do this also for Min=1
delete_words(_,[],R,R):-!.
delete_words(Min,[[Word,Count]|I],POSList,R):-
	Count < Min,!,
	delete_all(POSList,Word,POSList1),!,
	delete_words(Min,I,POSList1,R).
delete_words(Min,[_|I],POSList,R):-
	delete_words(Min,I,POSList,R).
	
calc_min(Min,NoWords,MinNo):-
	Min < 1,
        MinNo is  round(Min * NoWords),
        nl,write('Minimum Frequency is '),write(MinNo),
        write('  ('),write(Min),write('%)'),!.

calc_min(Min,NoWords,Min):-
        nl,write('Minimum Frequency is '),write(Min),
	PC is Min/NoWords,
        write('  ('),write(PC),write('%)'),!.

find_word_frequency(WL,[],WL):-!.
find_word_frequency(WL,[posSent(_,_,Word)|I],R):-
	member([Word,Count],WL),!,
	delete(WL,[Word,Count],WL1),
	NewCount is Count + 1,!,
	find_word_frequency([[Word,NewCount]|WL1],I,R).
find_word_frequency(WL,[posSent(_,_,Word)|I],R):-
	find_word_frequency([[Word,1]|WL],I,R).

delete_all([],_,[]):-!.
%never delete conPOS
delete_all([posSent(X,conPOS,Z)|POSList],Word,[posSent(X,conPOS,Z)|POSList1]):-!,
       delete_all(POSList,Word,POSList1).
%never delete numbers
delete_all([posSent(X,_,Z)|POSList],Word,[posSent(X,cdPOS,Z)|POSList1]):-
	number(Z),!,
       delete_all(POSList,Word,POSList1).
delete_all([posSent(_,_,Word)|POSList],Word,POSList1):-
	delete_all(POSList,Word,POSList1).
delete_all([posSent(X,Y,Z)|POSList],Word,[posSent(X,Y,Z)|POSList1]):-
	delete_all(POSList,Word,POSList1).



modify_posList(all,POSList,_,POSList):-!.

modify_posList(tgtOnly,POSList0,TGT,POSList):-
	create_posList_tgt_only(POSList0,TGT,POSList),!.

modify_posList(tgtPlus,POSList0,TGT,POSList):-
	create_posList(POSList0,TGT,POSList),!.

get_ontol_terms_arg2(TGT,Terms):-
        relation_under_test(R),
	get_ontol_terms_arg2_rep(R,TGT,Terms0),
	make_uniq(Terms0,Terms).


get_ontol_terms(TGT,Terms):-
        relation_under_test(R),
	get_ontol_terms_rep(R,TGT,Terms0),
	make_uniq(Terms0,Terms).

get_txt_terms(_,[]):-
	use_txt_mapping(false),!.

get_txt_terms(TGT,R):-
	use_txt_mapping(true),
        rep_get_txt_terms(TGT,R).

rep_get_txt_terms([],[]):-!.
rep_get_txt_terms([T|TGT],R):-
	findall(I,txtform_df(T,I),IL),!,
        rep_get_txt_terms(TGT,S),!,
        flatten(IL,ILL),
        append(ILL,S,R),!.
        


get_ontol_terms_arg2_rep(_,[],[]):-!.
get_ontol_terms_arg2_rep(R,[[_,[R,_,B]]|L],[B|S]):-
	get_ontol_terms_arg2_rep(R,L,S).
get_ontol_terms_arg2_rep(R,[_|L],S):-
	get_ontol_terms_arg2_rep(R,L,S).
	

get_ontol_terms_rep(_,[],[]):-!.
get_ontol_terms_rep(R,[[_,[R,A]]|L],[A|S]):-!,
	get_ontol_terms_rep(R,L,S).
get_ontol_terms_rep(R,[[_,[R,A,B]]|L],[A,B|S]):-
	get_ontol_terms_rep(R,L,S).
get_ontol_terms_rep(R,[_|L],S):-
	get_ontol_terms_rep(R,L,S).
	

% split input sentences on whether they contain a target reln,
% get the target terms, select additional sentences on whether
% they contain target terms, attempt to find another sentence
% for each target term

create_posList(POSList0,TGT,POSList):-
	get_sentences_with_tgt(TGT,TGTSent0,TGTTerms),
	make_uniq(TGTSent0,TGTSent1),
	nontgt_list(POSList0,TGTSent1,NONTGTSent0),
	make_uniq(NONTGTSent0,NONTGTSent1),
	posList_from_sentList(NONTGTSent1,POSList0,NONTGTposList0),
	sent_with_tgt_term(NONTGTposList0,TGTTerms,[],NONTGTSent2),
	make_uniq(NONTGTSent2,NONTGTSent3),
	posList_from_sentList(NONTGTSent3,POSList0,NONTGTposList),
	posList_from_sentList(TGTSent1,POSList0,TGTposList),
	append(TGTposList,NONTGTposList,POSList1),
	make_uniq(POSList1,POSList).

create_posList_tgt_only(POSList0,TGT,POSList):-
	get_sentences_with_tgt(TGT,TGTSent0,_),
	make_uniq(TGTSent0,TGTSent1),
	posList_from_sentList(TGTSent1,POSList0,POSList1),
	make_uniq(POSList1,POSList).
	
	


get_sentences_with_tgt([],[],[]):-!.
get_sentences_with_tgt([[S,[R,A]]|L],[S|T],[[A]|Terms]):-
	relation_under_test(R),
	get_sentences_with_tgt(L,T,Terms).
get_sentences_with_tgt([[S,[R,A,B]]|L],[S|T],[[A,B]|Terms]):-
	relation_under_test(R),
	get_sentences_with_tgt(L,T,Terms).
get_sentences_with_tgt([[S,[R,A,B,C]]|L],[S|T],[[A,B,C]|Terms]):-
	relation_under_test(R),
	get_sentences_with_tgt(L,T,Terms).
get_sentences_with_tgt([_|L],T,Terms):-
	get_sentences_with_tgt(L,T,Terms).


make_uniq([],[]):-!.
make_uniq([posSent(A,B,C)|J],K):-
	number(C),
	member(posSent(A,B,CC),J),
	number(CC),
	C =:= CC,!,
	make_uniq(J,K).
make_uniq([I|J],K):-
	member(I,J),!,
	make_uniq(J,K).
make_uniq([I|J],K):-
	number(I),
	member(II,J),
	number(II),
	I =:= II,!,
	make_uniq(J,K).
make_uniq([I|J],[I|K]):-
	make_uniq(J,K).

nontgt_list([],_,[]):-!.
nontgt_list([posSent(I,_,_)|J],L,M):-
	member(I,L),
	nontgt_list(J,L,M).
nontgt_list([posSent(I,_,_)|J],L,[I|M]):-
	nontgt_list(J,L,M).


posList_from_sentList([],_,[]):-!.
posList_from_sentList([S|T],AllPOS,R):-
	find_pos(S,AllPOS,SL),
	posList_from_sentList(T,AllPOS,R0),
	append(SL,R0,R).

find_pos(_,[],[]):-!.
find_pos(S,[posSent(S,A,B)|L],[posSent(S,A,B)|R]):-!,
	find_pos(S,L,R).
find_pos(S,[_|L],R):-
	find_pos(S,L,R).

sent_with_tgt_term(_,[],R,R):-!.
sent_with_tgt_term(POSList,[[A]|L],R,S):-
	member(ASent,R),
	member(posSent(ASent,_,A),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A]|L],R,S):-
	member(posSent(ASent,_,A),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A,B]|L],R,S):-
	member(ASent,R),
	member(posSent(ASent,_,A),POSList),
	member(posSent(ASent,_,B),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A,B]|L],R,S):-
	member(posSent(ASent,_,A),POSList),
	member(posSent(ASent,_,B),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A,B]|L],R,S):-
	sent_with_tgt_term(POSList,[[A],[B]|L],R,S).

sent_with_tgt_term(POSList,[[A,B,C]|L],R,S):-
	member(ASent,R),
	member(posSent(ASent,_,A),POSList),
	member(posSent(ASent,_,B),POSList),
	member(posSent(ASent,_,C),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A,B,C]|L],R,S):-
	member(posSent(ASent,_,A),POSList),
	member(posSent(ASent,_,B),POSList),
	member(posSent(ASent,_,C),POSList),
	sent_with_tgt_term(POSList,L,[ASent|R],S).

sent_with_tgt_term(POSList,[[A,B,C]|L],R,S):-
	sent_with_tgt_term(POSList,[[A,B],[A,C],[B,C]|L],R,S).

sent_with_tgt_term(POSList,[_|L],R,S):-
	sent_with_tgt_term(POSList,L,R,S).

write_neg_foil_tgts([]):- write('.'),nl.

write_neg_foil_tgts(R):-
	write(';'),nl,
	relation_under_test(REL),
	write_reln(REL,R),
	write('.'),nl.
	

getTypes(word,[],Ont,Ont):-!.
getTypes(_,[],_,[]):-!.
getTypes(sent,[posSent(S,_,_)|A],Ont,[S|R]):-
	getTypes(sent,A,Ont,R).
getTypes(pos,[posSent(_,S,_)|A],Ont,[S|R]):-
	getTypes(pos,A,Ont,R).
getTypes(word,[posSent(_,_,S)|A],Ont,[S|R]):-
	getTypes(word,A,Ont,R).

gen_result(I,L,L):-
	member(I,L),!.
gen_result(I,L,[I|L]):-!.




writeTypes(X,L):-
	write('#'),write(X),write(':'),writelist(L),write('.'),nl.
writeTypes_star(X,L):-
	write('#'),write(X),write(':'),writelist_star(L),write('.'),nl.

writelist([I]):-write(I).
writelist([I|J]):- write(I),write(','),writelist(J).

writelist_star([I]):-write('*'),write(I).
writelist_star([I|J]):- write('*'),write(I),write(','),writelist_star(J).

writeposent([]):-!.
writeposent([I|J]):-
	write(I),write('.'),nl,
	writeposent(J).


modify_sent_no([],[]):-!.
modify_sent_no([posSent(S,A,B)|L],[posSent(T,A,B)|R]):-!,
	concat(S,'001',T),
	modify_sent_no(L,R).
modify_sent_no([[S,A]|L],[[T,A]|R]):-!,
	concat(S,'001',T),
	modify_sent_no(L,R).


count_tgt_reln(_,[],0):-!.
count_tgt_reln(REL,[[_,[REL,_]]|L],R):-
	count_tgt_reln(REL,L,S),!,
	R is S + 1.
count_tgt_reln(REL,[[_,[REL,_,_]]|L],R):-
	count_tgt_reln(REL,L,S),!,
	R is S + 1.
count_tgt_reln(REL,[_|L],S):-
	count_tgt_reln(REL,L,S).


get_context([],[],[]):-!.


% was S,cdPOS,A
get_context([posSent(S,_,A),posSent(S,P1,B)|L],
	[context(S,A,B)|R],[B|W]):-
	relation_under_test(Rel),
	arg2(Rel,A),
	get_context([posSent(S,P1,B)|L],R,W).
get_context([_|L],R,W):-
	get_context(L,R,W).

check_context([],_,[]):-!.
check_context([context(S,A,B)|L],POSL,[context(S,A,B)|Res]):-
	member(posSent(_,_,B),POSL),!,
	check_context(L,POSL,Res).

check_context([_|L],POSL,Res):-
	check_context(L,POSL,Res).

/*
write_tgt_context([]):-!.
write_tgt_context([[_,[Rel,_,B]]|L]):-
	relation_under_test(Rel),
	write('1000,'),write(B),write(',?'),nl,
	write_tgt_context(L).
write_tgt_context([_|L]):-
	write_tgt_context(L).
*/



get_negated_tgts(TGT,POSL,TGTNeg,WDSNEG):-
	rep_neg_tgts(TGT,TGT,POSL,[],TGTNeg1,WDSNEG),!,
        rep_neg_tgts2(TGT,TGTNeg2),!,
        append(TGTNeg1,TGTNeg2,TGTNeg).

rep_neg_tgts([],_,_,R,R,[]):-!.

% was Rel,C,X/S,cdPOS,Y ???
rep_neg_tgts([[S,[Rel,_,X]]|TGTR],TGT,POSL,Res,Res0,[Y|WL]):-
	relation_under_test(Rel),
	member(posSent(S,C,Y),POSL),
	arg2(Rel,Y),
	not(member([S,[Rel,C,Y]],TGT)),
	not(member([S,[Rel,C,Y]],Res)),!,
	rep_neg_tgts([[S,[Rel,C,X]]|TGTR],TGT,POSL,[[S,[Rel,C,Y]]|Res],Res0,WL).
	
rep_neg_tgts([_|TGTR],TGT,POSL,Res,Res0,WL):-
	rep_neg_tgts(TGTR,TGT,POSL,Res,Res0,WL).
	

rep_neg_tgts2([],[]):-!.

rep_neg_tgts2([[_,[Rel,C,X]]|TGTR],[['1000',[Rel,C,X]]|Res]):-
	relation_under_test(Rel),
	rep_neg_tgts2(TGTR,Res).
	
rep_neg_tgts2([_|TGTR],Res):-
	rep_neg_tgts2(TGTR,Res).


arg2_vocab(POSL,VOC):-
	arg2_vocab_term(POSL,IL),!,
	make_uniq(IL,VOC).

arg2_vocab_term([],[]):-!.
arg2_vocab_term([posSent(_,_,Y)|POSL],[Y|R]):-
	relation_under_test(Rel),
	arg2(Rel,Y),
	arg2_vocab_term(POSL,R).

arg2_vocab_term([_|P],R):-
	arg2_vocab_term(P,R).


%------------------------------------------------------------------

% exhaustively enumerate the possible targets


enumerate_poss_tgt([],_,[]):-!.
enumerate_poss_tgt([A|Arg1],Arg2,Product):-
    tgt_enumerate_list(A,Arg2,P1),
    enumerate_poss_tgt(Arg1,Arg2,P2),!,
    append(P1,P2,Product).


tgt_enumerate_list(_,[],[]):-!.
tgt_enumerate_list(A,[B|Arg2],[[A,B]|R]):-
    tgt_enumerate_list(A,Arg2,R).

get_neg_tgt([],_,_,[]):-!.
get_neg_tgt([S|Sent],TGT,POSS,NTGT):-
    relation_under_test(Rel),    
    del_actual_tgt(S,Rel,TGT,POSS,NTGT0),
    gen_tgt_format(S,Rel,NTGT0,NTGT1),!,
    get_neg_tgt(Sent,TGT,POSS,NTGT2),
    append(NTGT1,NTGT2,NTGT),!.

del_actual_tgt(_,_,[],R,R):-!.
del_actual_tgt(S,REL,[[S,[REL,A,B]]|J],L,R):-!,
    delete(L,[A,B],LL),
    del_actual_tgt(S,REL,J,LL,R).
del_actual_tgt(S,REL,[_|J],L,R):-
    del_actual_tgt(S,REL,J,L,R).

gen_tgt_format(_,_,[],[]):-!.
gen_tgt_format(Sent,Rel,[[A,B]|TGT0],[[Sent,[Rel,A,B]]|TGT]):-
    gen_tgt_format(Sent,Rel,TGT0,TGT).




enumerate_poss_ntgt(TGT,L,NTGT):-
    enumerate_poss_tgt_3(L,L,L,P1),
    del_actual_tgt_ntgt(TGT,P1,NTGT),!.

enumerate_poss_tgt_3([],_,_,[]):-!.
enumerate_poss_tgt_3([A|Arg1],Arg2,Arg3,Product):-
    enumerate_poss_tgt(Arg2,Arg3,P1),
    relation_under_test(Rel),    
    gen_tgt_format(A,Rel,P1,P2),
    enumerate_poss_tgt_3(Arg1,Arg2,Arg3,P3),
    append(P2,P3,Product).


del_actual_tgt_ntgt([],R,R):-!.
del_actual_tgt_ntgt([[S,[REL,A,B]]|J],L,R):-!,
    delete(L,[S,[REL,A,B]],LL),
    del_actual_tgt_ntgt(J,LL,R).
del_actual_tgt_ntgt([_|J],L,R):-
    del_actual_tgt_ntgt(J,L,R).
