
%% parameters
%% File-name, 
%% Types: Sentence-numbers, POS, Words-in-texts, OntolArg1, OntolArg2, 
%% Target relations: Targets, 
%% Background relations (optional): Superclass, OntolTerms, Context, 
%% Background relation: POS-relations

write_ILP(FILE,
           Stype,POStype,Wtype,OntolArg1,OntolArg2,
           TGT,
           SUPER,OntolTerms,CONTEXT,
           POSList):-

	tell(FILE),  % the output file for FOIL

% FOIL requires types to be enumerated:
	writeTypes('SENT',['1000'|Stype]), % always add 1000 as this is the non-sentence
	writeTypes_star('POS',POStype),
	writeTypes_star('WORD',Wtype),
	writeTypes_star('ONT1',OntolArg1),
	writeTypes_star('ONT2',OntolArg2),nl,

% write the targets:
        write_foil_tgts(TGT),

        write(';'),nl,
        write_foil_neg_tgts(OntolArg1,OntolArg2,['1000'|Stype],TGT),

        write('.'),nl,

% background theories:
%  superclass from ontology
	if_add_ontology(SUPER), 

%  text mapping relations:
	if_req_writeontol_c(OntolTerms,TextList), 

%  context relation
	write_context(CONTEXT,OntolArg2),

%  the main pos relation representing the text
	writeposent_c(POSList),

% the `non-sentence' tuples for the pos relation
	add_nonsentence(OntolArg2,SUPER,TextList,POStype),
	write('.'),
	nl,
	told.


%-----------------------------------------------------

inverse_comptibility(A,A):-!.

%-----------------------------------------------------
% write the targets:


write_foil_tgts(R):-
	relation_under_test(REL),
	write(REL),
	write_args_of_reln,
	nl,
	write_reln(REL,R).

write_args_of_reln:- 
	relation_under_test_arity(unary),
	write('(SENT,ONT1)').

write_args_of_reln:- 
	relation_under_test_arity(binary),
	write('(SENT,ONT1,ONT2)').

write_reln(_,[]):-!.

write_reln(REL,[[Sent,[REL,X]]|L]):-
    write(Sent),write(','),
    write(X),
    nl,
    write_reln(REL,L).

write_reln(REL,[[Sent,[REL,X,Y]]|L]):-
    write(Sent),write(','),
    write(X),write(','),
    write(Y),nl,
    write_reln(REL,L).

write_reln(REL,[_|L]):-
    write_reln(REL,L).
    

write_foil_neg_tgts(OntolArg1,OntolArg2,Stype,TGT):-
	enumerate_poss_tgt(OntolArg1,OntolArg2,Product),
	get_neg_tgt(Stype,TGT,Product,NTGT),
	relation_under_test(REL),
	write_reln(REL,NTGT).


%-----------------------------------------------------
%  superclass from ontology

if_add_ontology([]):-!.
if_add_ontology(SUPER):-
	add_ontology(true),!,
	write('*superclass(ONT1,ONT1)'),nl,
	write_ontol(SUPER),nl.

if_add_ontology(_):-!.

write_ontol([]):-write('.'),!.
write_ontol([superclass(I,J)|K]):-
	write(I),write(','),write(J),nl,
	write_ontol(K).



%-----------------------------------------------------

%  text mapping relations:

if_req_writeontol_c(TGT,List):-
	use_txt_mapping(true),
        relation_under_test_concept(CON),
	writeontol_c([CON|TGT],List).

if_req_writeontol_c(_,[]).

writeontol_c(TGT,JL):-
	rep_get_txt_terms_2(TGT,JL),
	writeontol_binary_c(JL),
	writeontol_ternary_c(JL).
	
% get all the txtform_df() lists for each member of TGT
rep_get_txt_terms_2([],[]):-!.
rep_get_txt_terms_2([T|TGT],R):-
	findall(I,txtform_df(T,I),IL),!,
        rep_get_txt_terms_2(TGT,S),!,
        make_ont_list(T,IL,ILL),
        append(ILL,S,R).

make_ont_list(_,[],[]):-!.
make_ont_list(O,[I|J],[[O,I]|K]):-
    make_ont_list(O,J,K).

writeontol_binary_c([]):-!.
writeontol_binary_c(A):- write('*txtform1(ONT1,WORD)'),nl,
	writeontol_c_rep(binary,A),
	write('.'),nl.

writeontol_ternary_c([]):-!.
writeontol_ternary_c(A):- write('*txtform2(ONT1,WORD,WORD)'),nl,
	writeontol_c_rep(ternary,A),
	write('.'),nl.
	
writeontol_c_rep(_,[]):-!.
writeontol_c_rep(binary,[[A,[C]]|D]):-
	write(A),write(','),write(C),nl,
	writeontol_c_rep(binary,D).
writeontol_c_rep(ternary,[[A,[C,D]]|E]):-
	write(A),write(','),write(C),write(','),write(D),nl,
	writeontol_c_rep(ternary,E).
writeontol_c_rep(X,[_|D]):-
	writeontol_c_rep(X,D).


%-----------------------------------------------------

%  context relation

write_context(CL,TGT):-
	add_context(true),
	write_bin_context(CL,TGT),
	write_tern_context(CL),!.
write_context(_,_):-!.

write_bin_context(CL,_):-not(member(context(_,_,_),CL)),!.
write_bin_context(CL,TGT):-
	add_nonsentence(true),
	write('*context(SENT,WORD,WORD)'),nl,
	write_tgt_context(TGT),
	wr_context(2,CL),write('.'),nl.
write_bin_context(CL,_):-
	write('*context(SENT,WORD,WORD)'),nl,
	wr_context(2,CL),write('.'),nl.

write_tern_context(CL):-not(member(context(_,_,_,_),CL)),!.
write_tern_context(CL):-
	write('*context(SENT,WORD,WORD,WORD)'),nl,
	wr_context(3,CL),write('.'),nl.


wr_context(_,[]):-!.
wr_context(2,[context(A,B,C)|L]):-
	write(A),write('001,'),write(B),write(','),write(C),nl,
	wr_context(2,L).

wr_context(3,[context(A,B,C,D)|L]):-
	write(A),write('001,'),write(B),write(','),write(C),write(','),write(D),nl,
	wr_context(3,L).

wr_context(X,[_|L]):-
	wr_context(X,L).

write_tgt_context([]):-!.
write_tgt_context([B|L]):-
	write('1000,'),write(B),write(',?'),nl,
	write_tgt_context(L).


%-----------------------------------------------------

%  the main pos relation representing the text
	
writeposent_c([]):-!.
writeposent_c(L):- write('*posSent(SENT,POS,WORD)'),nl,writeposent_c_rep(L).
	
writeposent_c_rep([]):-!.
writeposent_c_rep([posSent(SENT,POS,WORD)|J]):-
	write(SENT),write(','),write(POS),write(','),write(WORD),nl,
	writeposent_c_rep(J).

%-----------------------------------------------------


% the `non-sentence' tuples for the pos relation

add_nonsentence(L,SUPER,TXT,POStype):-
    add_nonsentence(true),
    new_numerical_possent(L,SUPER,TXT,POStype).

add_nonsentence(_,_,_,_):-!.

new_numerical_possent([],SUPER,TXT,POStype):-
	use_txt_mapping(true),add_concepts(true),
	write('1000,v,be'),nl,
	wr_get_super(SUPER,SUPER1),
	make_uniq(SUPER1,SUPER2),
	wr_super('conPOS',SUPER2),
	wr_get_super(TXT,TXT1),
	make_uniq(TXT1,TXT2),
	wr_super(n,TXT2),
	wr_pos_nonsent(POStype).



new_numerical_possent([],_,TXT,POStype):-
	use_txt_mapping(true),
	write('1000,v,be'),nl,
	wr_get_super(TXT,SL),
	make_uniq(SL,SLL),
	wr_super(n,SLL),
	wr_pos_nonsent(POStype).

new_numerical_possent([],SUPER,_,POStype):-
	add_concepts(true),
	write('1000,v,be'),nl,
	wr_get_super(SUPER,SL),
	make_uniq(SL,SLL),
	wr_super('conPOS',SLL),
	wr_pos_nonsent(POStype).

new_numerical_possent([],_,_,POStype):-write('1000,v,be'),nl,
	wr_pos_nonsent(POStype).

new_numerical_possent([I|L],SUPER,TXT,POStype):-
	number(I),
	write('1000'),write(',cdPOS,'),write(I),nl,
	new_numerical_possent(L,SUPER,TXT,POStype).

new_numerical_possent([I|L],SUPER,TXT,POStype):-
	write('1000'),write(',j,'),write(I),nl,
	new_numerical_possent(L,SUPER,TXT,POStype).

wr_pos_nonsent([]):-!.
wr_pos_nonsent([I|IL]):-
    	write('1000,'),write(I),write(',?'),nl,
        wr_pos_nonsent(IL).

wr_get_super([],[]):-!.
wr_get_super([superclass(I,_)|L],[I|R]):-
	wr_get_super(L,R).

wr_get_super([[_,[I]]|L],[I|R]):-
	wr_get_super(L,R).

wr_get_super([[_,[I,J]]|L],[I,J|R]):-
	wr_get_super(L,R).

wr_super(_,[]):-!.
wr_super(X,[I|L]):-
	write('1000,'),write(X),write(','),write(I),nl,
	wr_super(X,L).


%-----------------------------------------------------

not_all_numerical(L):-
	member(A,L),
	not(is_number(A)).
