% generateILP_lime.pl

%% parameters
%% File-name, 
%% Types: Sentence-numbers, POS, Words-in-texts, OntolArg1, OntolArg2, 
%% Target relations: Targets, 
%% Background relations (optional): Superclass, OntolTerms, TextList, Context, 
%% Background relation: POS-relations

write_ILP(FILE,
           Stype,POStype,Wtype0,OntolArg10,OntolArg20,
           TGT0,
           SUPER0,OntolTerms,CONTEXT0,
           POSList0):-

	tell(FILE),  % the output file for LIME

% make all lists FOIL/LIME/GOLEM compatible
	make_compat(TGT0,TGT),
	make_compat(OntolArg10,OntolArg1),
	make_compat(OntolArg20,OntolArg2),
	make_compat(SUPER0,SUPER),
	make_compat(POSList0,POSList),
	make_compat(CONTEXT0,CONTEXT),
	make_compat(Wtype0,Wtype),!,

% write the targets first:
        write_lime_tgts(TGT),

% write the -ve targets:
	write_lime_neg_tgts2(TGT),

% recall lime does type inference !
% In LIME: write predicates for the constants (star-types in FOIL)
	write('BG'),nl,  % indicate that background kn. follows

% the small no words is a bit of a problem....

	write_consts_in_tgt(TGT),
	write_consts(Wtype),
/*
	write_consts(Wtype),
	write_consts(POStype),
	write_consts(OntolArg1),
	write_consts(OntolArg2),
        !,
	write_types(sent,Stype),
	write_nonsent_type,
	write_types(pos,POStype),
	write_types(word,Wtype),
*/


% background theories:
%  superclass from ontology
	if_add_ontology(SUPER), 

%  text mapping relations:
	if_req_writeontol_c(OntolTerms,TextList), 

%  context relation
	write_context(CONTEXT,OntolArg2,GensymList),

	write_types(sent,Stype),
        write_nonsent_type,!,
	append(GensymList,Wtype,Wtype1),
	write_types(word,Wtype1),
	write_types(ont1,OntolArg1),
	write_types(ont2,OntolArg2),

%  the main pos relation representing the text
	writeposent_c(POSList),

%  the `non-sentence' tuples for the pos relation
	add_nonsentence(OntolArg2,SUPER,TextList,GensymList),
	nl,nl,
	told.

%-----------------------------------------------------

make_compat([],[]):-!.

% just change the args of REL in target
make_compat([[Sent,[REL,X]]|J],[[Sent,[REL,K]]|L]):-!,
	make_word_compat(X,K),!,
	make_compat(J,L).
make_compat([[Sent,[REL,X,Y]]|J],[[Sent,[REL,K,KK]]|L]):-!,
	make_word_compat(X,K),!,
	make_word_compat(Y,KK),!,
	make_compat(J,L).

% changes arg in superclass
make_compat([superclass(X,Y)|J],[superclass(K,KK)|L]):-!,
	make_word_compat(X,K),!,
	make_word_compat(Y,KK),!,
	make_compat(J,L).
	
make_compat([posSent(A,B,C)|J],[posSent(A,K,KK)|L]):-!,
	make_word_compat(B,K),!,
	make_word_compat(C,KK),!,
	make_compat(J,L).

	
make_compat([context(A,B,C)|J],[context(A,K,KK)|L]):-!,
	make_word_compat(B,K),!,
	make_word_compat(C,KK),!,
	make_compat(J,L).

make_compat([A|J],[K|L]):-
	make_word_compat(A,K),!,
	make_compat(J,L).


make_word_compat(A,A):-
       integer(A),!.

make_word_compat(A,B):-
       number(A),!,
       concat('\'',A,B0),
       concat(B0,'\'',B),!.


make_word_compat(A,B):-
	name(A,AN),
	delete(AN,45,ANN),    % remove any -
	delete(ANN,95,ANNN),  % remove any _
	ontol_rename(ANNN,B). % construct allowable name

ontol_rename([X|A],B):-
	is_uppercase(X),!,
	name(B0,[X|A]),
	concat('so',B0,B),!.

ontol_rename([X|A],B):-
	is_number(X),
	not_all_numerical(A),
	name(B0,[X|A]),
        concat(no,B0,B).

ontol_rename(A,B):-
	name(B,A),!.

hyphenate_word([A|B],[A|C]):-
        is_number(A),!,
	hyphenate_word(B,C).

hyphenate_word(C,[45|C]):-!.

not_all_numerical(L):-
	member(A,L),
	not(is_number(A)).

%-----------------------------------------------------

inverse_comptibility(A,B):-
	name(A,[115, 111, X|Y]),  %soX|Y
	is_number(X),
	name(B,[X|Y]),!.

inverse_comptibility(A,A).
%-----------------------------------------------------
% write_consts

write_consts([]):-!.
write_consts([I|J]):-
	number(I),!,
	write(no),write(I),write('('),write(I),write(').'),nl,
	write_consts(J).

write_consts([I|J]):-
	write(I),write('('),write(I),write(').'),nl,
	write_consts(J).


number_or_quoted(I,I):-
	number(I),!.
number_or_quoted(I,J):-
	name(I,[39|L]),
	append(LL,[39],L),
	delete(LL,46,LLL),
	name(J,LLL),!.
	
	


write_consts_in_tgt(TGT):-
	get_Ont1_from_tgt(TGT,O1),
	get_Ont2_from_tgt(TGT,O2),
	make_uniq(O1,O11),
	make_uniq(O2,O22),
	write_consts_in_tgt_rep(O11),
	write_consts_in_tgt_rep(O22),!.

write_consts_in_tgt_rep([]):-!.
write_consts_in_tgt_rep([I|R]):-
	number(I),
	write(no),write(I),write('('),write(I),write(').'),nl,
	write_consts_in_tgt_rep(R).

write_consts_in_tgt_rep([I|R]):-
	write(I),write('('),write(I),write(').'),nl,
	write_consts_in_tgt_rep(R).


%-----------------------------------------------------

% write_types

write_types(_,[]):-!.
write_types(T,[I|J]):-
	write(T),write('('),write(I),write(').'),nl,
	write_types(T,J).


write_nonsent_type:-
	add_nonsentence(true),
	write('sent(1000).'),nl,!.
write_nonsent_type:-!.


%-----------------------------------------------------
% write the targets:


write_lime_tgts(R):-
	write('E+'),nl,
	write_lime_tgts_any(R).

write_lime_tgts_any(R):-
	relation_under_test(REL),
	write_reln(REL,R).

write_reln(_,[]):-!.
write_reln(REL,[[Sent,[REL,X]]|L]):-
    write(REL),write('('),write(Sent),write(','),write(X),write(').'),
    nl,
    write_reln(REL,L).

write_reln(REL,[[Sent,[REL,X,Y]]|L]):-
    write(REL),write('('),write(Sent),write(','),write(X),write(','),
    write(Y),write(').'),nl,
    write_reln(REL,L).

write_reln(REL,[_|L]):-
    write_reln(REL,L).
    

write_lime_neg_tgts(TGT,OntolArg2,Stype,TGT):-
	add_nonsentence(true),
	get_Ont1_from_tgt(TGT,OntolArg1),
	make_uniq(OntolArg1,OntolArg11),
	enumerate_poss_tgt(OntolArg11,OntolArg2,Product),
%        filter_neg_tgts(1,Product0,Product),
	get_neg_tgt([1000|Stype],TGT,Product,NTGT),
	write('E-'),nl,
	write_lime_tgts_any(NTGT).

write_lime_neg_tgts(TGT,OntolArg2,Stype,TGT):- 
	get_Ont1_from_tgt(TGT,OntolArg1),
	make_uniq(OntolArg1,OntolArg11),
	enumerate_poss_tgt(OntolArg11,OntolArg2,Product),
%        filter_neg_tgts(1,Product0,Product),

	get_neg_tgt(Stype,TGT,Product,NTGT),
	write('E-'),nl,
	write_lime_tgts_any(NTGT).

write_lime_neg_tgts2(TGT):-
	write('E-'),nl,!,
	rep_write_lime_neg_tgts2(TGT,TGT,[],R),!,
	duplic_neg_tgts(R,RR),!,
	write_lime_tgts_any(RR).

duplic_neg_tgts([],[]):-!.
duplic_neg_tgts([[S,[R,A,C]]|L],[[S,[R,A,C]],[S,[R,'soThing',C]]|Res]):-
	duplic_neg_tgts(L,Res).


rep_write_lime_neg_tgts2([],_,R,R):-!.

rep_write_lime_neg_tgts2([[S,[R,A,_]]|L],TGT,PS,[[1000,[R,A,C]]|Res]):-
	add_nonsentence(true),
	relation_under_test(R),
	member([_,[R,_,C]],TGT),
	not(member([S,[R,A,C]],TGT)),
	not(member([_,[R,_,C]],PS)),!,
	rep_write_lime_neg_tgts2(L,TGT,[[S,[R,A,C]]|PS],Res).

rep_write_lime_neg_tgts2([[S,[R,A,_]]|L],TGT,PS,[[1000,[R,A,C]]|Res]):-
	add_nonsentence(true),
	relation_under_test(R),
	member([_,[R,_,C]],TGT),
	not(member([S,[R,A,C]],TGT)),!,
	rep_write_lime_neg_tgts2(L,TGT,[[S,[R,A,C]]|PS],Res).

rep_write_lime_neg_tgts2([[S,[R,A,_]]|L],TGT,PS,Res):-
	relation_under_test(R),
	member([_,[R,_,C]],TGT),
	not(member([S,[R,A,C]],TGT)),
	not(member([_,[R,_,C]],PS)),!,
	rep_write_lime_neg_tgts2(L,TGT,[[S,[R,A,C]]|PS],Res).

rep_write_lime_neg_tgts2([[S,[R,A,_]]|L],TGT,PS,Res):-
	relation_under_test(R),
	member([_,[R,_,C]],TGT),
	not(member([S,[R,A,C]],TGT)),!,
	rep_write_lime_neg_tgts2(L,TGT,[[S,[R,A,C]]|PS],Res).
	
rep_write_lime_neg_tgts2([_|L],TGT,PS,Res):-
	rep_write_lime_neg_tgts2(L,TGT,PS,Res).
	

filter_neg_tgts(1,[X,_,_,_,_,_,_,_,_|L],[X|R]):-
       filter_neg_tgts(2,L,R).
filter_neg_tgts(2,[_,_,X,_,_,_,_,_|L],[X|R]):-
       filter_neg_tgts(3,L,R).
filter_neg_tgts(3,[_,_,_,_,_,_,X,_,_|L],[X|R]):-
       filter_neg_tgts(1,L,R).
filter_neg_tgts(_,L,L):-!.

get_Ont1_from_tgt([],[]):-!.
get_Ont1_from_tgt([[_,[R,A,_]]|TGT],[A|Res]):-
	relation_under_test(R),
	get_Ont1_from_tgt(TGT,Res).
get_Ont1_from_tgt([_|TGT],Res):-
	get_Ont1_from_tgt(TGT,Res).

get_Ont2_from_tgt([],[]):-!.
get_Ont2_from_tgt([[_,[R,_,A]]|TGT],[A|Res]):-
	relation_under_test(R),
	get_Ont2_from_tgt(TGT,Res).
get_Ont2_from_tgt([_|TGT],Res):-
	get_Ont2_from_tgt(TGT,Res).
	
%-----------------------------------------------------

add_words_to_ont2(Words,Ont2,NewOnt2):-
	length(Ont2,Len),
	selectN(Len,Words,SubW),
	append(SubW,Ont2,NewOnt2),!.
	
create_ntgt_list(S,O1,O2,L):-
    selectN(3,S,S1),
    selectN(3,O1,O11),
    selectN(3,O2,O22),
    append(S1,O11,R0),
    append(R0,O22,L),!.

selectN(0,_,[]):-!.
selectN(_,[],[]):-!.
selectN(I,[X|L],[X|R]):-
	J is I - 1,
	selectN(J,L,R).

%-----------------------------------------------------
%  superclass from ontology

if_add_ontology([]):-!.
if_add_ontology(SUPER):-
	add_ontology(true),!,
	write_ontol(SUPER),nl.

if_add_ontology(_):-!.

write_ontol([]):-!.
write_ontol([superclass(I,J)|K]):-
	write('superclass('),write(I),write(','),write(J),write(').'),nl,
	write_ontol(K).

%-----------------------------------------------------

%  text mapping relations:

if_req_writeontol_c(TGT,List):-
	use_txt_mapping(true),
        relation_under_test_concept(CON),
	make_word_compat(CON,CONC),
	writeontol_c([CONC|TGT],List).

if_req_writeontol_c(_,[]).

writeontol_c(TGT,JL):-
	rep_get_txt_terms_2(TGT,JL),
	writeontol_binary_c(JL),
	writeontol_ternary_c(JL).
	
% get all the txtform_df() lists for each member of TGT
rep_get_txt_terms_2([],[]):-!.
rep_get_txt_terms_2([T|TGT],R):-
	findall(I,txtform_df(T,I),IL),!,
        rep_get_txt_terms_2(TGT,S),!,
	make_word_compat(T,TCompat),
        make_ont_list(TCompat,IL,ILL),
        append(ILL,S,R).

% associate TGT term with txtform list
make_ont_list(_,[],[]):-!.
make_ont_list(O,[I|J],[[O,I]|K]):-
    make_ont_list(O,J,K).

writeontol_binary_c([]):-!.
writeontol_binary_c(A):- 
	writeontol_c_rep(binary,A),
	nl.

writeontol_ternary_c([]):-!.
writeontol_ternary_c(A):-
	writeontol_c_rep(ternary,A),
	nl.
	
writeontol_c_rep(_,[]):-!.
writeontol_c_rep(binary,[[A,[C]]|D]):-
	write('txtform1('),write(A),write(','),write(C),write(').'),nl,
	writeontol_c_rep(binary,D).
writeontol_c_rep(ternary,[[A,[C,D]]|E]):-
	write('txtform2('),write(A),write(','),write(C),write(','),write(D),write(').'),nl,
	writeontol_c_rep(ternary,E).
writeontol_c_rep(X,[_|D]):-
	writeontol_c_rep(X,D).


%-----------------------------------------------------

%  context relation

write_context(CL,TGT,GensymList):-
	add_context(true),
	write_bin_context(CL,TGT,GensymList),
	write_tern_context(CL),!.
write_context(_,_):-!.

write_bin_context(CL,_,[]):-not(member(context(_,_,_),CL)),!.
write_bin_context(CL,TGT,GensymList):-
	add_nonsentence(true),
	write_tgt_context(TGT,GensymList),
	wr_context(2,CL),nl.
write_bin_context(CL,_,[]):-
	wr_context(2,CL),nl.

write_tern_context(CL):-not(member(context(_,_,_,_),CL)),!.
write_tern_context(CL):-
	wr_context(3,CL),nl.


wr_context(_,[]):-!.
wr_context(2,[context(A,B,C)|L]):-
	write('context('),write(A),write('001,'),write(B),write(','),
	write(C),write(').'),nl,
	wr_context(2,L).

wr_context(3,[context(A,B,C,D)|L]):-
	write('context('),write(A),write('001,'),write(B),write(','),
	write(C),write(','),write(D),write(').'),nl,
	wr_context(3,L).

wr_context(X,[_|L]):-
	wr_context(X,L).

write_tgt_context([],[]):-!.
write_tgt_context([B|L],[SYM|R]):-
	gensym(a,SYM),
	write('context('),write('1000,'),write(B),write(','),write(SYM),write(').'),nl,
	write_tgt_context(L,R).


%-----------------------------------------------------

%  the main pos relation representing the text
	
writeposent_c([]):-!.
writeposent_c(L):- 
        writeposent_c_rep(L).
	
writeposent_c_rep([]):-!.
writeposent_c_rep([posSent(SENT,POS,WORD)|J]):-
	write('posSent('),write(SENT),write(','),write(POS),write(','),
	write(WORD),write(').'),nl,
	writeposent_c_rep(J).

%-----------------------------------------------------


% the `non-sentence' tuples for the pos relation

add_nonsentence(L,SUPER,TXT,POStype):-
    add_nonsentence(true),
    new_numerical_possent(L,SUPER,TXT,POStype).

add_nonsentence(_,_,_,_):-!.

new_numerical_possent([],SUPER,TXT,POStype):-
	use_txt_mapping(true),add_concepts(true),
	write('posSent('),write('1000,v,be'),write(').'),nl,
	wr_get_super(SUPER,SUPER1),
	make_uniq(SUPER1,SUPER2),
	wr_super('conPOS',SUPER2),
	wr_get_super(TXT,TXT1),
	make_uniq(TXT1,TXT2),
	wr_super(n,TXT2),
	wr_pos_nonsent(POStype).



new_numerical_possent([],_,TXT,POStype):-
	use_txt_mapping(true),
	write('posSent('),write('1000,v,be'),write(').'),nl,
	wr_get_super(TXT,SL),
	make_uniq(SL,SLL),
	wr_super(n,SLL),
	wr_pos_nonsent(POStype).

new_numerical_possent([],SUPER,_,POStype):-
	add_concepts(true),
	write('posSent('),write('1000,v,be'),write(').'),nl,
	wr_get_super(SUPER,SL),
	make_uniq(SL,SLL),
	wr_super('conPOS',SLL),
	wr_pos_nonsent(POStype).

new_numerical_possent([],_,_,POStype):-write('posSent('),write('1000,v,be'),write(').'),nl,
	wr_pos_nonsent(POStype).

new_numerical_possent([I|L],SUPER,TXT,POStype):-
	number(I),
	write('posSent('),write('1000'),write(',cdPOS,'),write(I),write(').'),nl,
	new_numerical_possent(L,SUPER,TXT,POStype).

new_numerical_possent([I|L],SUPER,TXT,POStype):-
	write('posSent('),write('1000'),write(',j,'),write(I),write(').'),nl,
	new_numerical_possent(L,SUPER,TXT,POStype).

%wr_pos_nonsent(_):-!.

wr_pos_nonsent([]):-!.
wr_pos_nonsent([I|IL]):-
    	write('posSent('),write('1000,n,'),write(I),write(').'),nl,
        wr_pos_nonsent(IL).

/* was
wr_pos_nonsent([]):-!.
wr_pos_nonsent([I|IL]):-
	gensym(a,SYM),
    	write('posSent('),write('1000,'),write(I),write(','),write(SYM),write(').'),nl,
        wr_pos_nonsent(IL).
*/
wr_get_super([],[]):-!.
wr_get_super([superclass(I,_)|L],[I|R]):-
	wr_get_super(L,R).

wr_get_super([[_,[I]]|L],[I|R]):-
	wr_get_super(L,R).

wr_get_super([[_,[I,J]]|L],[I,J|R]):-
	wr_get_super(L,R).

wr_super(_,[]):-!.
wr_super(X,[I|L]):-
	write('posSent('),write('1000,'),write(X),write(','),write(I),write(').'),nl,
	wr_super(X,L).


%-----------------------------------------------------

