
%  main.pl   Advanced Knowledge Technology 


%  usage    ?- [main].     load all files
%           ?- prepro_text. (or extract_text. tag_text. stem_text.)
%           ?- writeFOIL.  create a data file in FOIL format,
%                          using the coreference algorithm
%           ?- score.      calculate Recall and Precision




:- [xmlfilter],[read_Brill],[writeFOIL],
   [coreference-all],[resolve_ref],[assess],
   [nature_ontology],[rand],[clean_code].

:- dynamic posSent/3.

t:- [main],writeFOIL.

%------------------------------------------------------------
% settings:

% stem tokens
%apply_stemming(true).
apply_stemming(false).

% merge POS tags (used in both learning input generation and scoring)
% i.e. group all noun POS into `n'
% not is introduced as a POS to distinguish it from other RB (adverbs)
%merge_pos(false).  
merge_pos(true).  

% filter out POS, or not
filter_pos(true).
%filter_pos(false).


% retain these POS if filtering POS
% (can be used independently of merging but
%  then the whole set of POS types gets retained)
% note: filter_pos(false) == retain_pos(All-listed)

retain_pos(n).
retain_pos(j).
retain_pos(r).
retain_pos(v).
/*
%retain_pos(in_).
%retain_pos(to_).
%retain_pos(cd_).
%retain_pos(rp).
%retain_pos(ex).
%retain_pos(prp).
%retain_pos(dt).
%retain_pos(cc_).
%retain_pos(not_).
%retain_pos(md_).
*/

retain_pos(in_).
retain_pos(to_).
retain_pos(cd_).
retain_pos(rp).
retain_pos(ex).
retain_pos(prp).
retain_pos(dt).
retain_pos(cc_).
retain_pos(not_).
retain_pos(md_).

%retain_pos(sym).

retain_pos(P):-
	filter_pos(false),
	map_class(P,Q),
	retain_pos(Q).


% filter out words based on frequency of occurrence in texts
% as a % (default 0.007, or 7 occurrences in 1000 words for concept/1)
% OR an absolute value > = 1
% (previously, used 4 as the absolute value)
min_word_frequency(1).

% policy for learning_input - sentence selection
learning_input(all).     % use all sentences
% learning_input(tgtOnly). % use only sentences with a target
%learning_input(tgtPlus).  % use target sentences plus other sentences
                           % containing target terms, see writeFOIL.pl

% use the clustering coreference algorithm
% *** note: check the effect of stemming on coref. - seems benign
%apply_coreference(true).
apply_coreference(false).

% define the relation to be extracted and tested

%relation_under_test('changeInAtmosphericConcentration').
%relation_under_test_arity(binary).

%relation_under_test(isa).
%relation_under_test_arity(binary).

relation_under_test(concept).
relation_under_test_arity(unary).

% use the text to ontology relationship
%use_txt_mapping(true).
use_txt_mapping(false).

%------------------------------------------------------------

% define the location of the language data files 

subdirDATA('/hame/stuart/AKT/KnowX/Nature/').

% define the list of files to be processed - training and testing


file_list(testing,['nature-1',
                    'nature-2','nature-3',
		    'nature-4','nature-5','nature-6',
		    'nature-7','nature-8','nature-9',
		    'nature-10',
		    'nature-11','nature-12','nature-13',
		    'nature-14','nature-15',
                    'new-scientist-1',
                    'new-scientist-2','new-scientist-3',
		    'new-scientist-4','new-scientist-5','new-scientist-6',
		    'new-scientist-7','new-scientist-8','new-scientist-9',
		    'new-scientist-10',
		    'new-scientist-11','new-scientist-12','new-scientist-13',
		    'new-scientist-14','new-scientist-15']).


file_list(training,['nature-1',
                    'nature-2','nature-3',
		    'nature-4','nature-5','nature-6',
		    'nature-7','nature-8','nature-9',
		    'nature-10',
		    'nature-11','nature-12','nature-13',
		    'nature-14','nature-15',
                    'new-scientist-1',
                    'new-scientist-2','new-scientist-3',
		    'new-scientist-4','new-scientist-5','new-scientist-6',
		    'new-scientist-7','new-scientist-8','new-scientist-9',
		    'new-scientist-10',
		    'new-scientist-11','new-scientist-12','new-scientist-13',
		    'new-scientist-14','new-scientist-15']).


% other useful lists

/*

file_list(training,['nature-1','nature-2','nature-3',
		    'nature-4','nature-5','nature-6',
		    'nature-7','nature-8','nature-9',
		    'nature-10']). /*,
		    'nature-11','nature-12','nature-13',
		    'nature-14','nature-15','nature-16',
		    'nature-17']). */

file_list(testing,['nature-1','nature-2','nature-3',
		    'nature-4','nature-5','nature-6',
		    'nature-7','nature-8','nature-9',
		    'nature-10']). /*
		    'nature-11','nature-12','nature-13',
		    'nature-14','nature-15','nature-16',
		    'nature-17']). */


file_list(training,['cadmium','californium','cesium',
	   'curium', 'calcium','cerium','chromium']):-!.

file_list(testing,['cadmium','californium','cesium',
	   'curium', 'calcium','cerium','chromium']):-!.


list(['actinium','aluminum','argon','arsenic','astatine','barium','beryllium','boron','cadmium','calcium','carbon','cesium','chlorine','chromium','cobalt','copper','curium','erbium','francium','germanium','gold']).

list(['holium','hydrogen','indium','iodine','iron','krypton','lithium','magnesium','mercury','neodymium','neon','nickel','nitrogen','oxygen','platinum','polonium','potassium','praseodymium','proctactinium','promethium','radium','radon','rhenium','rhodium','rubidium','samarium','silicone','silver','sodium','sulfer','tantalum','tellurium','terbium','tin','titanium','uranium','xenon','yttrium','zinc']).

list(['actinium','aluminum','argon','arsenic','astatine','barium','beryllium','boron','cadmium','calcium','carbon','cesium','chlorine','chromium','cobalt','copper','curium','erbium','francium','germanium','gold','holium','hydrogen','indium','iodine','iron','krypton','lithium','magnesium','mercury','neodymium','neon','nickel','nitrogen','oxygen','platinum','polonium','potassium','praseodymium','proctactinium','promethium','radium','radon','rhenium','rhodium','rubidium','samarium','silicone','silver','sodium','sulfer','tantalum','tellurium','terbium','tin','titanium','uranium','xenon','yttrium','zinc']).

*/
