
%  nature_ontology.pl

%%%%%%%%%%%%%%% for ILP %%%%%%%%%%%%%%
% define the relationship between an ontology concept
% and its likely textual form, where these are not the same
txtform_df('GreenhouseGas',[greenhouse,gas]).
txtform_df('GreenhouseEffect',[greenhouse,effect]).
txtform_df('GreenhouseWarming',[greenhouse,warm]).
txtform_df('WaterVapour',[water,vapour]).
txtform_df('CarbonDioxide',[carbon,dioxide]).
txtform_df('CarbonDioxide',[co2]).
txtform_df('Methane',[methane]).
txtform_df('Methane',[ch4]).
txtform_df('OxygenGas',[oxygen]).
txtform_df('Oxygen',[oxygen]).
txtform_df('NitrousOxide',[nitrogen,oxide]).
txtform_df('NitrousOxide',[n2o]).

txtform_df('CarbonMonoxide',[carbon,monoxide]).
txtform_df('CarbonMonoxide',[co]).
txtform_df('NaturalGas',[natural,gas]).
txtform_df('NitrogenGas',[nitrogen]).

txtform_df('CarbonTetrachloride',[carbon,tetrachloride]). 
txtform_df('Chlorodifluoromethane',[chlorodifluoromethane]).
txtform_df('ChlorofluoroCarbon',[chlorofluorocarbon]).
txtform_df('Fluoroform',[fluoroform]).
txtform_df('MethylChloroform',[methyl,chloroform]).
txtform_df('Ozone',[ozone]).
txtform_df('Perfluoroethane',[perfluoroethane]).
txtform_df('SulphurHexafluoride',[sulphur,hexafluoride]).
%txtform_df('TrifluoromethylSulphurPentafluoride',
%            [trifluoromethyl,sulphur,pentafluoride]).


/*txtform_df('High',[high]).
txtform_df('Low',[low]).
txtform_df('Increase',[increase]).
txtform_df('Decrease',[decrease]).
txtform_df('None',[none]). */

txtform_df('AtmosphericConcentration',[atmosphere,concentration]).
txtform_df('AtmosphericConcentration',[atmospheric,concentration]).


txtform1(X,Y):- txtform_df(X,[Y]).
txtform2(X,Y,Z):- txtform_df(X,[Y,Z]).

occurring_subclass(Top,List,Res):-
	interm_superclass_list(Top,Classes),
	rep_occurring_subclass_1(Classes,List,Res).

rep_occurring_subclass_1([],_,[]):-!.
rep_occurring_subclass_1([I|J],List,Res):-
	occurring_subclass_1(I,List,Res0),!,
	rep_occurring_subclass_1(J,List,Res1),
	append(Res0,Res1,Res),!.


occurring_subclass_1(_,[],[]):-!.
occurring_subclass_1(Gas,[posSent(_,conPOS,Arg1)|List],
	[superclass(Arg1,Gas)|Gases]):-
    superclass_rec(Arg1,Gas),!,
    occurring_subclass_1(Gas,List,Gases).

occurring_subclass_1(Gas,[[_,[RELN,Arg1,_]]|List],
	[superclass(Arg1,Gas)|Gases]):-
    relation_under_test(RELN),
    superclass_rec(Arg1,Gas),!,
    occurring_subclass_1(Gas,List,Gases).

occurring_subclass_1(Gas,[_|List],Gases):-
    occurring_subclass_1(Gas,List,Gases).


superclass_rec(Sub,Super):- superclass(Sub,Super),!.
superclass_rec(Sub1,Super):- superclass(Sub,Super),
    superclass_rec(Sub1,Sub).

interm_superclass_list(Top,[Top|ICL]):-
    findall(CL,interm_superclass(CL,Top),ICL).

interm_superclass(Cl,Top):-
	superclass_rec(Cl,Top),
	superclass(_,Cl).

occurring_symbol(_,[],[]):-!.
occurring_symbol(R,[[_,[_,Arg2]]|List],[arg2(R,Arg2)|GL]):-
    occurring_symbol(R,List,GL).


named_entity_POS([],[]):-!.

% two consec. words in sent A
named_entity_POS([posSent(A,POS1,X),posSent(A,POS2,Y)|L],S):-
    findall(CONCEPT,txtform_df(CONCEPT,[X,Y]),CONCEPTList),
    make_POS_cn(A,CONCEPTList,POSL),
    advance_or(POSL,[posSent(A,POS1,X),posSent(A,POS2,Y)|L],POSL1,M),
    named_entity_POS(M,R),!,
    append(POSL1,R,S).

% sent differs
named_entity_POS([_|L],M):- 
    named_entity_POS(L,M).
    


% no matching Concepts - get those for 1st word, advance 1
advance_or([],[posSent(A,_,X),posSent(A,POS2,Y)|L],POSL,
	   [posSent(A,POS2,Y)|L]):-!,
    findall(CONCEPT,txtform_df(CONCEPT,[X]),CONCEPTList),
    make_POS_cn(A,CONCEPTList,POSL),!.

% some matching Concepts - advance 2
advance_or(POSL,[_,_|L],POSL,L):-!.


%named_entity_POS([_|L],R):-
%    named_entity_POS(L,R).

advance_or_not([],L,L):-!.
advance_or_not(_,[_,_|L],L):-!.



make_POS_cn(_,[],[]):-!.
make_POS_cn(S,[A|B],[posSent(S,conPOS,A)|R]):-
    make_POS_cn(S,B,R).

%%%%%%%%%%%%%%% for coreference %%%%%%%%%%%%%%
% resolve_ref requires is_ontology_term/1
% to identify all terms in the ontology

is_ontology_term(X):- 
	class(X).

% coreference requires: 

% class_in_ontology/1
class_in_ontology(Word):- 
	class(Word).

%superclass_in_ontology/1
superclass_in_ontology(A):- 
	superclass(_,A).

% class_superclass_in_ontology/2
class_superclass_in_ontology(Word, Elementclass):- 
	superclass(Word,Elementclass).

% abbreviation/1
abbreviation_in_ontology(Word):- 
	abbreviation(Word,_).

% abbrev_class_in_ontology/2
abbrev_class_in_ontology(Symbol, Word):- 
	abbreviation(Symbol,Word).

class(X):- superclass(_,X).
class(X):- superclass(X,_).

superclass('AtmosphericConcentration','Thing').
superclass('Oxygen','Element').
superclass('Carbon','Element').

superclass('GreenhouseGas','Gas').
superclass('CarbonMonoxide','Gas').
superclass('NaturalGas','Gas').
superclass('NitrogenGas','Gas').
superclass('OxygenGas','Gas').
superclass('WaterVapour','Gas').

superclass('CarbonDioxide','GreenhouseGas'). 
superclass('CarbonTetrachloride','GreenhouseGas'). 
superclass('Chlorodifluoromethane','GreenhouseGas'). 
superclass('ChlorofluoroCarbon','GreenhouseGas'). 
superclass('Fluoroform','GreenhouseGas'). 
superclass('Methane','GreenhouseGas'). 
superclass('MethylChloroform','GreenhouseGas'). 
superclass('NitrousOxide','GreenhouseGas'). 
superclass('Ozone','GreenhouseGas'). 
superclass('Perfluoroethane','GreenhouseGas'). 
superclass('SulphurHexafluoride','GreenhouseGas'). 
superclass('TrifluoromethylSulphurPentafluoride','GreenhouseGas').

arg2('atmosphericConcentration_Qual',high).
arg2('atmosphericConcentration_Qual',low).

arg2('changeInAtmosphericConcentration',increase).
arg2('changeInAtmosphericConcentration',decrease).
arg2('changeInAtmosphericConcentration',none).

arg2('atmosphericConcentration',I):-number(I),I>1,I<1800.
arg2('atmosphericConcentration',I):-number(I),I>2200,I<1000000.

arg2_list('atmosphericConcentration_Qual',_,[high,low]).
arg2_list('changeInAtmosphericConcentration',_,[increase,decrease,none]).
arg2_list('atmosphericConcentration',I,I).

ontol_terms_list(['Thing','AtmosphericConcentration','Gas',
	'Oxygen', 'Carbon', 'GreenhouseGas', 'CarbonMonoxide', 'NaturalGas', 
	'NitrogenGas', 'OxygenGas', 'WaterVapour', 'CarbonDioxide', 'CarbonTetrachloride', 
	'Chlorodifluoromethane', 'ChlorofluoroCarbon', 'Fluoroform', 'Methane', 'MethylChloroform',
	'NitrousOxide', 'Ozone', 'Perfluoroethane', 'SulphurHexafluoride', 
	'TrifluoromethylSulphurPentafluoride']). 



abbreviation(co2,co2).
abbreviation(ch4,ch4).
abbreviation(n2o,n2o).



%---------------------------------------------
% namedata.pl would provide these:

name('John').
name('Smith').

% wordnet would provide these:
hyp(a,a).
s(a,a,a,a,a,a).

