
% read_Brill_ne.pl
% code for reading-in the Brill-processed text files
% output is posSent/3 list




go(FILESET,RESULT,WordFList):-
    merge_pos(true),
    filter_pos(true),!,
    file_list(FILESET,LIST),
    nl,write('Processing Files: '),
    readRepeat(1,LIST,RESULT00),!,
    misc_word_changes(RESULT00,RESULT0),
    merge_pos(RESULT0,RESULT1),
    retain_pos(RESULT1,RESULT2),
    find_word_frequency([],RESULT0,WordFList),
    add_concepts_to_POS(RESULT2,RESULT),!.

go(FILESET,RESULT,WordFList):-
    merge_pos(true),
    filter_pos(false),!,
    file_list(FILESET,LIST),
    nl,write('Processing Files: '),
    readRepeat(1,LIST,RESULT00),!,
    misc_word_changes(RESULT00,RESULT0),
    merge_pos(RESULT0,RESULT2),
    find_word_frequency([],RESULT0,WordFList),
    add_concepts_to_POS(RESULT2,RESULT),!.

go(FILESET,RESULT,WordFList):-
    merge_pos(false),!,
    file_list(FILESET,LIST),
    nl,write('Processing Files: '),
    readRepeat(1,LIST,RESULT0),!,
    misc_word_changes(RESULT0,RESULT2),
    find_word_frequency([],RESULT2,WordFList),
    add_concepts_to_POS(RESULT2,RESULT),!.

add_concepts_to_POS(L,M):-
    add_concepts(true),
    named_entity_POS(L,NE),
    append(NE,L,M),!.

add_concepts_to_POS(L,L):-!.
    

% mark not as a part of speech
% and other word level changes
misc_word_changes([],[]):-!.
misc_word_changes([posSent(A,_,not)|I],[posSent(A,not_,not)|J]):-
    misc_word_changes(I,J).
misc_word_changes([posSent(A,POS,'\%')|I],[posSent(A,POS,'percentageSymbol')|J]):-
    misc_word_changes(I,J).
misc_word_changes([posSent(A,POS,'\\')|I],[posSent(A,POS,'fsSymbol')|J]):-
    misc_word_changes(I,J).
misc_word_changes([posSent(A,POS,'\/')|I],[posSent(A,POS,'bsSymbol')|J]):-
    misc_word_changes(I,J).
misc_word_changes([posSent(A,X,B)|I],[posSent(A,X,B)|J]):-
    misc_word_changes(I,J).

merge_pos([],[]):-!.
merge_pos([posSent(A,X,B)|I],[posSent(A,Y,B)|J]):-
    map_class(X,Y),
    merge_pos(I,J).


retain_pos([],[]):-!.
retain_pos([posSent(A,X,B)|I],[posSent(A,X,B)|J]):-
    retain_pos(X),
    retain_pos(I,J).
retain_pos([_|I],J):-
    retain_pos(I,J).

%retain_pos(n).
%retain_pos(v).
%retain_pos(j).
%retain_pos(cd_).

%retain_pos(P):-
%	filter_pos(false),
%	map_class(P,Q),
%	retain_pos(Q).

map_class(nn,n).
map_class(nnp,n).
map_class(nnps,n).
map_class(nns,n).
map_class(np,n).
map_class(nps,n).
map_class(jj,j).
map_class(jjr,j).
map_class(jjs,j).
map_class(jjss,j).
map_class(r,r).
map_class(rb,r).
map_class(rbr,r).
map_class(rbs,r).
map_class(vb,v).
map_class(vbd,v).
map_class(vbg,v).
map_class(vbn,v).
map_class(vbp,v).
map_class(vbz,v).
map_class(in,inPOS).
map_class(to,toPOS).
map_class(cd,cdPOS).
map_class(md,mdPOS).
map_class('.',sym).
map_class(':',sym).
map_class(',',sym).
map_class(wp,prp).
map_class('wp$',prp).
map_class('wrb',r).
map_class('wdt',dt).
map_class(rp,rp).
map_class(ex,ex).
map_class('prp',prp).
map_class('prp$',prp).
map_class('prpr$',prp).
map_class(pdt,dt).
map_class(dt,dt).
map_class(cc,ccPOS).
map_class(sym,sym).
map_class(not_,notPOS).
map_class(_,sym). 




readRepeat(_,[],[]):-!.
readRepeat(CurrentNo,[File|L],Result):-
    apply_coreference(true),
    nl,write(' '),write(File),write(';'),
    read_brill_file(File,POSList,CurrentNo,NextNo),!,
    cluster(POSList,POSList1,NounClasses),!,
    resolve_ref(POSList1,NounClasses,POSList1,POSList2),!,
    nl,write_coref(POSList1,POSList2),!,
    readRepeat(NextNo,L,ResultRest),!,
    append(POSList2,ResultRest,Result).

readRepeat(CurrentNo,[File|L],Result):-
    apply_coreference(false),
%    nl,write(' '),write(File),write(';'),
    read_brill_file(File,POSList,CurrentNo,NextNo),!,
    changeCase(POSList,POSList1),
    readRepeat(NextNo,L,ResultRest),!,
    append(POSList1,ResultRest,Result).

read_brill_file(File,POSList,CurrentNo,NextNo):-
	apply_stemming(false),
	read_brill_no_stem_file(File,POSList,CurrentNo,NextNo).

read_brill_file(File,POSList,CurrentNo,NextNo):-
	apply_stemming(true),
	read_brill_stem_file(File,POSList,CurrentNo,NextNo).


read_brill_no_stem_file(File,POSList,CurrentNo,NextNo):-  
	concat(File,'.tag',FileTag),
        subdirDATA(DIR),
        concat(DIR,FileTag,TaggedFile),
	readin_xml(TaggedFile,Content0),
	fix_end_of_sent_tag(Content0,Content1),
	make_POS_list(CurrentNo,Content1,POSList,NextNo).

read_brill_stem_file(File,POSList,CurrentNo,NextNo):-  
	concat(File,'.tag',FileTag),
        subdirDATA(DIR),
        concat(DIR,FileTag,TaggedFile),
	readin_xml(TaggedFile,Content0), 
	concat(File,'.stem',FileStem),
        subdirDATA(DIR),
        concat(DIR,FileStem,StemmedFile),
	readin_xml(StemmedFile,ContentStem),
	fix_end_of_sent_tag(Content0,Content1),
	make_POS_stem_list(CurrentNo,Content1,ContentStem,POSList,NextNo).

	
fix_end_of_sent_tag([I,['/',Tag]],[L]):-
	append(I,['/',Tag],L).
	
fix_end_of_sent_tag([I,['/',Tag]|R],[K|L]):-
	append(I,['/',Tag],K),
	fix_end_of_sent_tag(R,L).

make_POS_list(LastNo,[],[],LastNo):-!.
make_POS_list(CurrentNo,[H|Content],POSList,LastNo):-
	make_POS_sent(CurrentNo,H,List1),
	NextNo is CurrentNo + 1,
	make_POS_list(NextNo,Content,List2,LastNo),
	append(List1,List2,POSList).

make_POS_sent(_,[],[]):-!.
make_POS_sent(Sent,[Token,'/',POS|L],[posSent(Sent,POS,Token)|R]):-
	make_POS_sent(Sent,L,R).


make_POS_stem_list(LastNo,[],[],[],LastNo):-!.
make_POS_stem_list(CurrentNo,[H|Content],[HStem|ContentStem],POSList,LastNo):-
	make_POS_stem_sent(CurrentNo,H,HStem,List1),
	NextNo is CurrentNo + 1,
	make_POS_stem_list(NextNo,Content,ContentStem,List2,LastNo),
	append(List1,List2,POSList).

make_POS_stem_sent(_,[],[],[]):-!.
make_POS_stem_sent(Sent,[_,'/',POS|L],[Token|StemL],
	[posSent(Sent,POS,Token)|R]):-
%	write(TokenOld),write('>'),write(Token),nl,
	make_POS_stem_sent(Sent,L,StemL,R).



renum(X,[posSent(Y,A,B)|L],R):-
    Z is Y - X,
    renum_rec(Z,[posSent(Y,A,B)|L],R).

renum_rec(_,[],[]):-!.
renum_rec(Z,[posSent(Y,A,B)|L],[posSent(W,A,B)|M]):-
    W is Y - Z,
    renum_rec(Z,L,M).


%%%% re-numbering 
renum(X,[posSent(Y,A,B)|L],R,NextNo):-
    Z is Y - X,
    renum_rec(Z,[posSent(Y,A,B)|L],R,NextNo).

renum_rec(Z,[posSent(Y,A,B)],[posSent(W,A,B)],NextNo):-
    W is Y - Z,
    NextNo is W + 1.
renum_rec(Z,[posSent(Y,A,B)|L],[posSent(W,A,B)|M],NextNo):-
    W is Y - Z,
    renum_rec(Z,L,M,NextNo).


	

lowercase(A,B):-
	name(A,AN),
	to_lowercase(AN,BN),
	name(B,BN).

to_lowercase([],[]):-!.
to_lowercase([X|L],[Y|M]):-
    is_uppercase(X),
    Y is X + 32,
    to_lowercase(L,M).

to_lowercase([X|L],[X|M]):-
    to_lowercase(L,M).

is_uppercase(X):-
    X > 64,
    X < 91.




