
% xmlfilter.pl
% remove and get xml from data files


% the file list and directory information should be 
% used to generate file names in readGATE and writeFOIL
% i.e. used concat as below

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main predicates

%extract_target/1 get xml
extract_target(FILES,R):-
	file_list(FILES,L),
	rep_extract_target(1,L,R).
%	arg2_conversion(RR,R).
%	capitalise(RR,R).

% prepro_text/0 apply all text preprocessing algorithms
prepro_text:- 
	extract_text,!,
	tag_text,!,
	stem_text.

% extract_text/0 remove xml
extract_text:-
	file_list_all(L),
	rep_extract(L),
	nl,
	write('Text generated').

% tag_text/0 apply Brill tagger to text
tag_text:-
	cd('./BrillTagger'),
	file_list_all(L),
	rep_tag(L),
	cd('..'),
	nl,
	write('Tagging complete').

% stem_text/0 apply Humpherys/Carroll morph analyser to text
stem_text:-
	cd('./Morph'),
	file_list_all(L),
	rep_stem(L),
	cd('..'),
	nl,
	write('Stemming complete').


%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rep_extract([]):-!.
rep_extract([I|J]):-
	concat(I,'.xml',II), %chemtext.xml
        subdirDATA(D), 
        concat(D,II,III),  
	readin_xml(III,K),
	concat(I,'.txt',IO),
        concat(D,IO,GIO),
	write_out_text(GIO,K),!,
	rep_extract(J).
	
rep_extract_target(_,[],[]):-!.
rep_extract_target(C,[I|J],Res):-
	concat(I,'.xml',II),
        subdirDATA(Dir), 
        concat(Dir,II,III), 
	readin_xml(III,K),
	rep_filter_text(C,K,L,D),
	rep_extract_target(D,J,R),
	append(L,R,Res).
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rep_tag([]):-!.
rep_tag([I|J]):-
	concat(I,'.txt',II), 
        subdirDATA(D), 
        concat(D,II,III),  
	concat(I,'.tag',IO),
        concat(D,IO,GIO),
        concat('./tagger LEXICON ',III, C1),
        concat(C1,' BIGRAMS LEXICALRULEFILE CONTEXTUALRULEFILE > ',C2),
	concat(C2,GIO,Command),
	write(Command),nl,!,
	shell(Command,_),!,
	rep_tag(J).



rep_stem([]):-!.
rep_stem([I|J]):-
	concat(I,'.txt',II), 
        subdirDATA(D), 
        concat(D,II,III),  
	concat(I,'.stem',IO),
        concat(D,IO,GIO),
        concat('./morpha -u < ',III, C1),
        concat(C1,'  > ',C2),
	concat(C2,GIO,Command),
	write(Command),nl,!,
	shell(Command,_),!,
	rep_stem(J).



readin_xml(IP,P):-
    see(IP),
    read_to_eof_xml(O),
    seen,
    fix_errors(O,P).

write_out_text(File,P):-
	tell(File),
	rep_filter_xml(P,Q),
	write_text(Q),nl,
	told.

get_xml(P,Q):-
	rep_filter_text(1,P,Q,_).


rep_filter_xml([],[]):-!.
rep_filter_xml([I|J],[II|R]):-
	filter_xml(I,II),
	rep_filter_xml(J,R).

filter_xml([],[]):-!.
filter_xml(['<','target',_,'(',_,')',_,'>','<','/','target','>'|L],R):-
	filter_xml(L,R),!.
filter_xml(['<','target',_,'(',_,_,_,')',_,'>','<','/','target','>'|L],R):-
	filter_xml(L,R),!.
filter_xml([I|J],[I|R]):-!,
	filter_xml(J,R).

rep_filter_text(X,[],[],X):-!.
rep_filter_text(X,[I|J],Res,XX):-
	filter_text(X,I,II),
	Y is X + 1,
	rep_filter_text(Y,J,R,XX),
	append(II,R,Res).

filter_text(_,[],[]):-!.

filter_text(I,['<','target',X0,'(',X1,')',_,'>','<','/','target','>'|L],
	[[I,[X0N,X1]]|R]):-
	name(X0,[_,_,_,_,_,_|X0L]),
	name(X0N,X0L),
	filter_text(I,L,R),!.
filter_text(I,['<','target',X0,'(',X1,',',X2,')',_,'>','<','/','target','>'|L],
	[[I,[X0N,X1,X2]]|R]):-
	name(X0,[_,_,_,_,_,_|X0L]),
	name(X0N,X0L),
	filter_text(I,L,R),!.
filter_text(I,[_|J],R):-!,
	filter_text(I,J,R).


extract_pred(I,K):-
	name(I,[_,_,_,_,_,_|L]),
	reverse(L,[_|M]),
	tok_pred(M,[],J),
	reverse(J,K).

tok_pred([],L,[N]):-name(N,L),!.
tok_pred([Z|L],[],R):-
	tok_pred_sym(Z),
	tok_pred(L,[],R).
tok_pred([Z|L],A,[AW|R]):-
	tok_pred_sym(Z),
	name(AW,A),
	tok_pred(L,[],R).
tok_pred([X|L],A,R):-
	tok_pred(L,[X|A],R).

tok_pred_sym(40).
tok_pred_sym(41).
tok_pred_sym(44).

% tokenising is a bit simple - fix errors
fix_errors([],[]):-!.
fix_errors([[]|L],R):-fix_errors(L,R),!.

% fix tokenising of numbers problem
fix_errors([I,[J|JL]|K],O):-
    name(J,[JJ|_]),
    is_number(JJ),
    reverse(I,['.',II|_]),
    number(II),
    append(I,[J|JL],M),
    join_up('.',M,N),
    fix_errors([N|K],O).

fix_errors([I,['g','.'],KK|K],O):-
    reverse(I,['.','e'|_]),
    append(I,['g'|KK],M),
    join_up('.',M,N),
    fix_errors([N|K],O).


fix_errors([I,J|K],O):-
    reverse(I,['.',X|_]),
    name(X,[XL]),
    is_uppercase(XL),
    append(I,J,M),
    fix_errors([M|K],O).

fix_errors([I|L],[I|R]):-
    fix_errors(L,R).

join_up(S,[A,S,B|L],[AAA|L]):-
    concat(A,S,AA),
    concat(AA,B,AAA),!.

join_up(S,[I|L],[I|R]):-
    join_up(S,L,R).

is_number(I):- I > 47, I < 58,!.
    
read_to_eof_xml(I):-
        get0(C),
        read_lines_xml(C,I),!.

read_lines_xml(-1,[]):-!. %  EOF
read_lines_xml(C,[Line|L]):-
        read_line_xml(C,Line,D),!,
        read_lines_xml(D,L),!.

read_line_xml(-1,[],-1). %  EOF
read_line_xml(A,[W],D):- stopword(A,W),get0(D),!.
read_line_xml(C,[Word|Line],L):-
	word_xml(C,Word),
	get0(D),
        read_line_xml(D,Line,L),!.
read_line_xml(C,Result,L):-
        read_word_xml(C,Word,D),!,
        read_line_xml(D,Line,L),!,
        check_name_xml(Word,Line,Result).

check_name_xml([],L,L):-!. % don't add '' to output
check_name_xml(W,L,[S|L]):-
	name(S,W).

read_word_xml(-1,[],-1):-!. %  EOF
read_word_xml(A,[],A):- word_xml(A,_).
read_word_xml(A,[],A):- stopword(A,_).
read_word_xml(10,[],D):- get0(D),!. % nl
read_word_xml(32,[],D):- get0(D),!. % space
read_word_xml(C,[C|I],L):-
        get0(D),
        read_word_xml(D,I,L),!.


word_xml(37,'%').
word_xml(40,'(').
word_xml(41,')').
word_xml(44,',').
word_xml(47,'/').
word_xml(60,'<').
word_xml(62,'>').
stopword(46,'.'). % .
stopword(33,'!'). % !
stopword(63,'?'). % ? 


write_text([]):-!.
write_text([I|J]):-
	write_line(I),nl,
	write_text(J).

write_line([I,'.']):-write(I),write(' .'),!.
write_line([]):-!.
write_line([I,II|J]):-
	no_skip(II),!,
	write(I),write(II),
	write_line(J).
write_line([I|J]):-
	write(I),write(' '),
	write_line(J).

%no_skip(',').
no_skip('.').

concat_list([],''):-!.
concat_list([I|J],R):-
	concat_list(J,S),
	concat(I,S,R).

file_list_all(I):-
	file_list(training,A),
	file_list(testing,B),
	append(A,B,C),
	make_uniq(C,I),!.


capitalise([],[]):-!.
capitalise([[S,[R,A,B]]|L],[[S,[R,A,BB]]|RL]):-
	name(B,[X|BN]),
	Y is X - 32,
	name(BB,[Y|BN]),!,
	capitalise(L,RL).
capitalise([A|L],[A|R]):-
	capitalise(L,R).
	
arg2_conversion([],[]):-!.

arg2_conversion([[S,[R,_,B]]|L],[[S,[RR,B]]|RL]):-
	reln_arg2_name(R,RR),
	arg2_conversion(L,RL).

arg2_conversion([_|L],RL):-
	arg2_conversion(L,RL).
